@@ -4,7 +4,7 @@ use crate::executor::{Execute, Executor};
 use crate::pool::PoolOptions;
 use crate::pool::{Pool, PoolConnection};
 use crate::postgres::message::{MessageFormat, Notification};
-use crate::postgres::{PgConnection, PgDone, PgRow, PgStatement, PgTypeInfo, Postgres};
+use crate::postgres::{PgConnection, PgQueryResult, PgRow, PgStatement, PgTypeInfo, Postgres};
 use either::Either;
 use futures_channel::mpsc;
 use futures_core::future::BoxFuture;
@@ -142,7 +142,7 @@ impl PgListener {
     ///
     /// As notifications are transient, any received while the connection was lost, will not
     /// be returned. If you'd prefer the reconnection to be explicit and have a chance to
-    /// do something before, please see [`try_recv`].
+    /// do something before, please see [`try_recv`](Self::try_recv).
     ///
     /// # Example
     ///
@@ -162,8 +162,6 @@ impl PgListener {
     /// # Ok(())
     /// # }).unwrap();
     /// ```
-    ///
-    /// [`try_recv`]: #method.try_recv
     pub async fn recv(&mut self) -> Result<PgNotification, Error> {
         loop {
             if let Some(notification) = self.try_recv().await? {
@@ -248,9 +246,8 @@ impl PgListener {
     ///
     /// The backing connection will be automatically reconnected should it be lost.
     ///
-    /// This has the same potential drawbacks as [`recv`].
+    /// This has the same potential drawbacks as [`recv`](PgListener::recv).
     ///
-    /// [`recv`]: #method.recv
     pub fn into_stream(mut self) -> impl Stream<Item = Result<PgNotification, Error>> + Unpin {
         Box::pin(try_stream! {
             loop {
@@ -266,7 +263,7 @@ impl<'c> Executor<'c> for &'c mut PgListener {
     fn fetch_many<'e, 'q: 'e, E: 'q>(
         self,
         query: E,
-    ) -> BoxStream<'e, Result<Either<PgDone, PgRow>, Error>>
+    ) -> BoxStream<'e, Result<Either<PgQueryResult, PgRow>, Error>>
     where
         'c: 'e,
         E: Execute<'q, Self::Database>,
